/*
 * main.c - fraktaly: kapradina
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#define __USE_SVID
#define __USE_GNU

#include <stdio.h>
#include <unistd.h>
#include <math.h>

#define DEBUG 0
#include "debug.h"

#define DELAY		0
#define LEVEL_MAX	8
#define BASE_SIZE	(1/(float)LEVEL_MAX)

enum { UP, LEFT, RIGHT, DOWN };

int iter = 0, iter_max = 1;

void draw_fern(int level, int x, int y, int size, float orient) {
	float p;

	iter++;
	p = 100 * ((float)iter / (float)iter_max);

	/* vypocet souradnic */
	int new_x  = x + cos(orient / 180. * M_PI) * size;
	int new_y  = y - sin(orient / 180. * M_PI) * size;
	
	/* prostredni */
	//printf("set_line_width %d\n", level > 3 ? 1 : 3);
	printf("draw_line %d %d %d %d\n", x, y, new_x, new_y);
	printf("progressbar_position %1.0f\n", p);
	printf("progressbar_text %d / %d (%1.0f %%)\n", iter, iter_max, p);

	//if (level < 6)
	//	fflush(stdout);
	//usleep(DELAY);

	/* 3 okolo */
	if (level++ <= LEVEL_MAX) {
		draw_fern(level, new_x, new_y, size * 0.9, orient + 2);
		draw_fern(level, new_x, new_y, size * 0.4, orient - 80);
		draw_fern(level, new_x, new_y, size * 0.4, orient + 80);
	}

}

int main (void) {
	int size = 768;

	printf(
		"set_fullscreen 1\n"
		"set_windowtitle Fraktaly: Sierpinského kobereček\n"
		"set_size %d %d\n"
		"set_line_width 1\n"
		"progressbar_show\n"
		"clear\n"
		"set_color 0 24580 0\n",
		size, size
	);
	fflush(stdout);

	iter_max = 1.5 * pow(3, LEVEL_MAX);
	draw_fern(1, size/2, size - size * BASE_SIZE, size * BASE_SIZE, 90);

	printf("progressbar_hide\n");

	return 0;
}

/* vim:set encoding=utf8: */
