/*
 * main.c - fraktaly: strom
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdio.h>
#include <unistd.h>

#define DEBUG 0
#include "debug.h"

#define DELAY 5e3

//      00   01   10     11
enum { LEFT, UP, RIGHT, DOWN };
enum { X, Y };

void draw_tree(int level, int x, int y, int size, int orient) {
	int new_size = size / 3;
	int top_l[2], top_r[2], center[2];
	float scale = 1./2.;

	int center_sig[2][4] = {
		/* X */ { -1,  0, +1,  0 },
		/* Y */ {  0, -1,  0, +1 }
	};
	int top_l_sig[2][4] = {
		/* X */ { -1, -1, +1, +1 },
		/* Y */ { +1, -1, -1, +1 }
	};
	int top_r_sig[2][4] = {
		/* X */ { -1, +1, +1, -1 },
		/* Y */ { -1, -1, +1, +1 }
	};

	/* vypocet vrcholu */
	center[X] = x + center_sig[X][orient] * size;
	center[Y] = y + center_sig[Y][orient] * size;
	top_l[X] = center[X] + top_l_sig[X][orient] * size * scale;
	top_r[X] = center[X] + top_r_sig[X][orient] * size * scale;
	top_l[Y] = center[Y] + top_l_sig[Y][orient] * size * scale;
	top_r[Y] = center[Y] + top_r_sig[Y][orient] * size * scale;

	/* prostredni */
	printf("draw_line %d %d %d %d\n", x, y, center[X], center[Y]);
	printf("draw_line %d %d %d %d\n", center[X], center[Y], top_l[X], top_l[Y]);
	printf("draw_line %d %d %d %d\n", center[X], center[Y], top_r[X], top_r[Y]);
	fflush(stdout);
	usleep(DELAY);

	/* vetve */
	if (level++ <= 4) {
		draw_tree(level, top_l[X], top_l[Y], new_size, orient);
		draw_tree(level, top_l[X], top_l[Y], new_size, (orient + 3) % 4);
		draw_tree(level, top_r[X], top_r[Y], new_size, orient);
		draw_tree(level, top_r[X], top_r[Y], new_size, (orient + 1) % 4);
	}
}

int main (void) {
	int size = 768;

	printf(
		"set_fullscreen 1\n"
		"set_windowtitle Fraktaly: Strom\n"
		"set_size %d %d\n"
		"set_line_width 3\n"
		"clear\n"
		"set_color 0 0 32768\n",
		size, size
	);
	fflush(stdout);

	draw_tree(1, size / 2, size * 3./4., size / 4, UP);

	return 0;
}

/* vim:set encoding=utf8: */
